﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace OutOfMemoryException
{
    public partial class OutOfMemoryException : Form
    {
        private BindingSource _bindingSource;  
        private DataSet    _dataSet  = new DataSet();
        private DataTable _dtTable1 = new DataTable("Table1");
        private DataTable _dtTable2 = new DataTable("Table2");
        private DataTable _dtTable3 = new DataTable("Table3");
        private DataTable _dtTable4 = new DataTable("Table4");

        public OutOfMemoryException()
        {
            InitializeComponent();
        }

        private void OutOfMemoryException_Load(object sender, EventArgs e)
        {
            SetUpDataSource();
        }

        private void SetUpDataSource()
        {
            _dtTable1.Columns.Add("Key1");
            _dtTable1.Columns.Add("1_Column 1");
            _dtTable1.Columns.Add("1_Column 2");
            _dtTable1.Columns.Add("1_Column 3");
            _dtTable1.Columns.Add("1_Column 4");

            _dtTable2.Columns.Add("Key1");
            _dtTable2.Columns.Add("Key2");
            _dtTable2.Columns.Add("2_Column 1");
            _dtTable2.Columns.Add("2_Column 2");
            _dtTable2.Columns.Add("2_Column 3");

            _dtTable3.Columns.Add("Key1");
            _dtTable3.Columns.Add("Key2");
            _dtTable3.Columns.Add("Key3");
            _dtTable3.Columns.Add("2_Column 1");
            _dtTable3.Columns.Add("2_Column 2");
            _dtTable3.Columns.Add("2_Column 3");

            _dtTable4.Columns.Add("Key1");
            _dtTable4.Columns.Add("Key2");
            _dtTable4.Columns.Add("Key3");
            _dtTable4.Columns.Add("Key4");
            _dtTable4.Columns.Add("2_Column 1");
            _dtTable4.Columns.Add("2_Column 2");
            _dtTable4.Columns.Add("2_Column 3");
          
            DataRow dRow;
            // Parent rows
            for (int x = 0; x < 800; x++)
            {
                dRow = _dtTable1.Rows.Add(new object[] { "Row" + x.ToString(), "Row" + x.ToString() + " Col 1", "Row" + x.ToString() + " Col 2", "Row" + x.ToString() + " Col 3", "Row" + x.ToString() + " Col 4" });
            }
          
            // Child rows
            for (int x = 0; x < 800; x++)
            {
                dRow = _dtTable2.Rows.Add(new object[] { "Row" + x.ToString(), "2Row" + x.ToString(), "2Row" + x.ToString() + " Col 1", "2Row" + x.ToString() + " Col 2", "2Row" + x.ToString() + " Col 3" });
            }

            // Grandchild rows
            for (int x = 0; x < 800; x++)
            {
                dRow = _dtTable3.Rows.Add(new object[] { "Row" + x.ToString(), "2Row" + x.ToString(), "3Row" + x.ToString(), "3Row" + x.ToString() + " Col 1", "3Row" + x.ToString() + " Col 2", "3Row" + x.ToString() + " Col 3" });
            }

            _dataSet.Tables.Add(_dtTable1);
            _dataSet.Tables.Add(_dtTable2);
            _dataSet.Tables.Add(_dtTable3);

            _dataSet.Relations.Add("T1-T2", _dataSet.Tables["Table1"].Columns["Key1"], _dataSet.Tables["Table2"].Columns["Key1"]);

            _dataSet.Relations.Add("T2-T3", new DataColumn[]  {_dataSet.Tables["Table2"].Columns["Key1"], _dataSet.Tables["Table2"].Columns["Key2"] },
                   new DataColumn[] { _dataSet.Tables["Table3"].Columns["Key1"], _dataSet.Tables["Table3"].Columns["Key2"] }, true);

            // Great Grandchild rows; these are an oversimplification of our application rows where the number of rows and columns can vary which is 
            // why each one is its own band
            for (int x = 0; x < 800; x++)
            {
                DataTable dt = _dtTable4.Clone();
                dt.TableName = "T-" + x.ToString();  
                DataRow dr = dt.Rows.Add(new object[] { "Row" + x.ToString(), "2Row" + x.ToString(), "3Row" + x.ToString(), "4Row" + x.ToString(), "4Row" + x.ToString() + " Col 1", "4Row" + x.ToString() + " Col 2", "4Row" + x.ToString() + " Col 3" });
                _dataSet.Tables.Add(dt);

                _dataSet.Relations.Add(dt.TableName, new DataColumn[] { _dataSet.Tables["Table3"].Columns["Key1"], _dataSet.Tables["Table3"].Columns["Key2"], _dataSet.Tables["Table3"].Columns["Key3"] },
                       new DataColumn[] { _dataSet.Tables[dt.TableName].Columns["Key1"], _dataSet.Tables[dt.TableName].Columns["Key2"], _dataSet.Tables[dt.TableName].Columns["Key3"] }, true);
            }

            _bindingSource = new BindingSource(_dataSet, "");
            ultraGrid.DataSource = _bindingSource;
        }

        private void ExpandAll_Click(object sender, EventArgs e)
        {
            foreach (UltraGridRow row in ultraGrid.Rows)
            {
                row.ExpandAll();
            }
        }

        private void CollapseAll_Click(object sender, EventArgs e)
        {
            foreach (UltraGridRow row in ultraGrid.Rows)
            {
                row.CollapseAll();
            }
        }

        private void SetActivation_Click(object sender, EventArgs e)
        {
            foreach (UltraGridRow row in ultraGrid.Rows)
            {
                SetRowActivation(row, Activation.NoEdit);
                row.Selected = true;
            }
        }

        private void SetRowActivation(UltraGridRow aRow, Activation aActivation)
        {
            try
            {
                
                aRow.Activation = aActivation;

                if (aRow.HasChild(false))
                {
                    UltraGridRow cRow = aRow.GetChild(ChildRow.First);
                    while (cRow != null)
                    {
                        SetRowActivation(cRow, aActivation);
                        cRow = cRow.GetSibling(SiblingRow.Next, true, false);
                    }
                }
            }
            catch  
            {
                throw;
            }
        }
    }
}
